/* ide-popover-positioner.h
 *
 * Copyright 2022 Christian Hergert <chergert@redhat.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <gtk/gtk.h>

#include <libide-core.h>

G_BEGIN_DECLS

#define IDE_TYPE_POPOVER_POSITIONER (ide_popover_positioner_get_type())

IDE_AVAILABLE_IN_ALL
G_DECLARE_INTERFACE (IdePopoverPositioner, ide_popover_positioner, IDE, POPOVER_POSITIONER, GtkWidget)

struct _IdePopoverPositionerInterface
{
  GTypeInterface parent_iface;

  void (*present) (IdePopoverPositioner *self,
                   GtkPopover           *popover,
                   GtkWidget            *relative_to,
                   const GdkRectangle   *pointing_to);
};

IDE_AVAILABLE_IN_ALL
void ide_popover_positioner_present (IdePopoverPositioner *self,
                                     GtkPopover           *popover,
                                     GtkWidget            *relative_to,
                                     const GdkRectangle   *pointing_to);

G_END_DECLS
